"""Constants and enums used by the Ezviz Cloud API wrapper.

Includes default timeouts, request headers used to emulate the mobile
client, and a large collection of enums that map integers/strings from
the Ezviz API to descriptive names.
"""

from enum import Enum, unique

from .utils import generate_unique_code

FEATURE_CODE = generate_unique_code()
XOR_KEY = b"\x0c\x0eJ^X\x15@Rr"
DEFAULT_TIMEOUT = 25
MAX_RETRIES = 3
REQUEST_HEADER = {
    "featureCode": FEATURE_CODE,
    "clientType": "3",
    "osVersion": "",
    "clientVersion": "",
    "netType": "WIFI",
    "customno": "1000001",
    "ssid": "",
    "clientNo": "web_site",
    "appId": "ys7",
    "language": "en_GB",
    "lang": "en",
    "sessionId": "",
    "User-Agent": "okhttp/3.12.1",
}  # Standard android header.
MQTT_APP_KEY = "4c6b3cc2-b5eb-4813-a592-612c1374c1fe"
APP_SECRET = "17454517-cc1c-42b3-a845-99b4a15dd3e6"


@unique
class MessageFilterType(Enum):
    """Message filter types for unified list."""

    FILTER_TYPE_MOTION = 2402
    FILTER_TYPE_PERSON = 2403
    FILTER_TYPE_VEHICLE = 2404
    FILTER_TYPE_SOUND = 2405
    FILTER_TYPE_ALL_ALARM = 2401
    FILTER_TYPE_SYSTEM_MESSAGE = 2101


@unique
class DeviceSwitchType(Enum):
    """Device switch name and number."""

    ALARM_TONE = 1
    STREAM_ADAPTIVE = 2
    LIGHT = 3
    INTELLIGENT_ANALYSIS = 4
    LOG_UPLOAD = 5
    DEFENCE_PLAN = 6
    PRIVACY = 7
    SOUND_LOCALIZATION = 8
    CRUISE = 9
    INFRARED_LIGHT = 10
    WIFI = 11
    WIFI_MARKETING = 12
    WIFI_LIGHT = 13
    PLUG = 14
    SLEEP = 21
    SOUND = 22
    BABY_CARE = 23
    LOGO = 24
    MOBILE_TRACKING = 25
    CHANNELOFFLINE = 26
    ALL_DAY_VIDEO = 29
    AUTO_SLEEP = 32
    ROAMING_STATUS = 34
    DEVICE_4G = 35
    ALARM_REMIND_MODE = 37
    OUTDOOR_RINGING_SOUND = 39
    INTELLIGENT_PQ_SWITCH = 40
    DOORBELL_TALK = 101
    HUMAN_INTELLIGENT_DETECTION = 200
    LIGHT_FLICKER = 301
    ALARM_LIGHT = 303
    ALARM_LIGHT_RELEVANCE = 305
    DEVICE_HUMAN_RELATE_LIGHT = 41
    TAMPER_ALARM = 306
    DETECTION_TYPE = 451
    OUTLET_RECOVER = 600
    CHIME_INDICATOR_LIGHT = 611
    TRACKING = 650
    CRUISE_TRACKING = 651
    PARTIAL_IMAGE_OPTIMIZE = 700
    FEATURE_TRACKING = 701


@unique
class SupportExt(Enum):
    """Supported device extensions."""

    SupportAHeadWakeupWifi = 696
    SupportAITag = 522
    SupportAbsenceReminder = 181
    SupportActiveDefense = 96
    SupportAddAfterGuide = 737
    SupportAddDelDetector = 19
    SupportAddSmartChildDev = 156
    SupportAlarmInterval = 406
    SupportAlarmLight = 113
    SupportAlarmVoice = 7
    SupportAlertDelaySetup = 383
    SupportAlertTone = 215
    SupportAntiOpen = 213
    SupportApMode = 106
    SupportAssociateDoorlockOnline = 415
    SupportAudioCollect = 165
    SupportAudioConfigApn = 695
    SupportAudioOnoff = 63
    SupportAutoAdjust = 45
    SupportAutoOffline = 8
    SupportAutoSleep = 144
    SupportBackLight = 303
    SupportBackendLinkIpcStream = 596
    SupportBatteryDeviceP2p = 336
    SupportBatteryManage = 119
    SupportBatteryNonPerOperateP2p = 417
    SupportBatteryNumber = 322
    SupportBellSet = 164
    SupportBleConfigApn = 759
    SupportBluetooth = 58
    SupportBlutoothWifiConfig = 710
    SupportBodyFaceFilter = 318
    SupportBodyFaceMarker = 319
    SupportCall = 191
    SupportCapture = 14
    SupportChanType = 52
    SupportChangeSafePasswd = 15
    SupportChangeVoice = 481
    SupportChangeVolume = 203
    SupportChannelOffline = 70
    SupportChannelTalk = 192
    SupportChime = 115
    SupportChimeDoorbellAutolink = 334
    SupportChimeIndicatorLight = 186
    SupportCloseInfraredLight = 48
    SupportCloud = 11
    SupportCloudVersion = 12
    SupportConcealResource = 286
    SupportCorrelationAlarm = 387
    SupportCruiseTraking = 197
    SupportCustomVoice = 92
    SupportCustomVoicePlan = 222
    SupportDayNightSwitch = 238
    SupportDdns = 175
    SupportDecivePowerMessage = 218
    SupportDecouplingAlarmVoice = 473
    SupportDefaultVoice = 202
    SupportDefence = 1
    SupportDefencePlan = 3
    SupportDelayLocalRecord = 636
    SupportDetectHumanCar = 224
    SupportDetectMoveHumanCar = 302
    SupportDevOfflineAlarm = 450
    SupportDeviceAutoVideoLevel = 729
    SupportDeviceEmptyPasswordSetting = 609
    SupportDeviceGetAccountPermission = 568
    SupportDeviceIntrusionDetection = 385
    SupportDeviceLinkDevice = 593
    SupportDeviceLocation = 745
    SupportDevicePermissionType = 659
    SupportDeviceRevisionSetting = 499
    SupportDeviceRfSignalReport = 325
    SupportDeviceRing = 185
    SupportDeviceTransboundaryDetection = 386
    SupportDevicelog = 216
    SupportDisk = 4
    SupportDiskBlackList = 367
    SupportDistributionNetworkBetweenDevice = 420
    SupportDisturbMode = 217
    SupportDisturbNewMode = 292
    SupportDoorCallPlayBack = 545
    SupportDoorCallQuickReply = 544
    SupportDoorLookStateShow = 690
    SupportDoorbellIndicatorLight = 242
    SupportDoorbellTalk = 101
    SupportEcdhV2 = 519
    SupportEmojiInteraction = 573
    SupportEnStandard = 235
    SupportEncrypt = 9
    SupportEnterCardDetail = 779
    SupportEventVideo = 393
    SupportExtremePowerSaving = 827
    SupportEzvizChime = 380
    SupportFaceFrameMark = 196
    SupportFeatureTrack = 321
    SupportFecCeilingCorrectType = 312
    SupportFecDeskTopCorrectType = 666
    SupportFecWallCorrectType = 313
    SupportFilter = 360
    SupportFishEye = 91
    SupportFlashLamp = 496
    SupportFlowStatistics = 53
    SupportFocusAdjust = 817
    SupportFullDayRecord = 88
    SupportFullScreenPtz = 81
    SupportGetDeviceAuthCode = 492
    SupportHorizontalPanoramic = 95
    SupportHostScreen = 240
    SupportIndicatorBrightness = 188
    SupportIndicatorLightDay = 331
    SupportIntellectualHumanFace = 351
    SupportIntelligentNightVisionDuration = 353
    SupportIntelligentPQSwitch = 366
    SupportIntelligentTrack = 73
    SupportInterconnectionDbChime = 550
    SupportIpcLink = 20
    SupportIsapi = 145
    SupportKeyFocus = 74
    SupportKindsP2pMode = 566
    SupportLANPort = 769
    SupportLanguage = 47
    SupportLaserPtCtrl = 640
    SupportLightAbilityRemind = 301
    SupportLightRelate = 297
    SupportLocalConnect = 507
    SupportLocalLockGate = 662
    SupportLockConfigWay = 679
    SupportMessage = 6
    SupportMicroVolumnSet = 77
    SupportMicroscope = 60
    SupportModifyChanName = 49
    SupportModifyDetectorguard = 23
    SupportModifyDetectorname = 21
    SupportMore = 54
    SupportMotionDetection = 97
    SupportMultiChannelFlip = 732
    SupportMultiChannelSharedService = 720
    SupportMultiChannelType = 719
    SupportMultiScreen = 17
    SupportMultiSubsys = 255
    SupportMultilensPlay = 665
    SupportMusic = 67
    SupportMusicPlay = 602
    SupportNatPass = 84
    SupportNeedOpenMode = 754
    SupportNetProtect = 290
    SupportNewSearchRecords = 256
    SupportNewTalk = 87
    SupportNewWorkMode = 687
    SupportNightVisionMode = 206
    SupportNoencriptViaAntProxy = 79
    SupportNvrEncrypt = 465
    SupportOneClickReset = 738
    SupportOneKeyPatrol = 571
    SupportOpticalZoom = 644
    SupportOsd = 153
    SupportPaging = 249
    SupportPanoramaPicListSize = 731
    SupportPartAreaRecord = 615
    SupportPartialImageOptimize = 221
    SupportPetHomeCharge = 805
    SupportPetPlayPath = 801
    SupportPetTalkChangeVoice = 639
    SupportPicInPic = 460
    SupportPirDetect = 100
    SupportPirSetting = 118
    SupportPlaybackAsyn = 375
    SupportPlaybackMaxSpeed = 610
    SupportPlaybackPiP = 645
    SupportPlaybackQualityChange = 200
    SupportPlaybackSmallSpeed = 585
    SupportPointLocateView = 724
    SupportPoundSignShow = 699
    SupportPoweroffRecovery = 189
    SupportPreP2P = 59
    SupportPreset = 34
    SupportPresetAlarm = 72
    SupportPreviewCorrectionInOldWay = 581
    SupportPreviewNoPlayback = 780
    SupportProtectionMode = 64
    SupportPtz = 154
    SupportPtz45Degree = 32
    SupportPtzCenterMirror = 37
    SupportPtzCommonCruise = 35
    SupportPtzFigureCruise = 36
    SupportPtzFocus = 99
    SupportPtzHorizontal360 = 199
    SupportPtzLeftRight = 31
    SupportPtzLeftRightMirror = 38
    SupportPtzManualCtrl = 586
    SupportPtzModel = 50
    SupportPtzNew = 605
    SupportPtzPrivacy = 40
    SupportPtzTopBottom = 30
    SupportPtzTopBottomMirror = 39
    SupportPtzZoom = 33
    SupportPtzcmdViaP2pv3 = 169
    SupportQosTalkVersion = 287
    SupportQualityDisable = 660
    SupportQuickplayWay = 149
    SupportRateLimit = 65
    SupportRebootDevice = 452
    SupportRegularBrightnessPlan = 384
    SupportRelatedDevice = 26
    SupportRelatedStorage = 27
    SupportRelationCamera = 117
    SupportRemindAudition = 434
    SupportRemoteAuthRandcode = 28
    SupportRemoteControl = 800
    SupportRemoteOpenDoor = 592
    SupportRemoteQuiet = 55
    SupportRemoteUnlock = 648
    SupportReplayChanNums = 94
    SupportReplayDownload = 260
    SupportReplaySpeed = 68
    SupportResolution = 16
    SupportRestartTime = 103
    SupportReverseDirect = 69
    SupportRingingSoundSelect = 241
    SupportSafeModePlan = 22
    SupportSdCover = 483
    SupportSdHideRecord = 600
    SupportSdkTransport = 29
    SupportSeekPlayback = 257
    SupportSensibilityAdjust = 61
    SupportServerSideEncryption = 261
    SupportSetWireioType = 205
    SupportSignalAsyn = 183
    SupportSignalCheck = 535
    SupportSimCard = 194
    SupportSleep = 62
    SupportSmartBodyDetect = 244
    SupportSmartNightVision = 274
    SupportSoundLightAlarm = 214
    SupportSsl = 25
    SupportStopRecordVideo = 219
    SupportSwitchLog = 187
    SupportSwitchTalkmode = 170
    SupportTalk = 2
    SupportTalkType = 51
    SupportTalkVolumeAdj = 455
    SupportTamperAlarm = 327
    SupportTearFilm = 454
    SupportTemperatureAlarm = 76
    SupportTextToVoice = 574
    SupportTimeSchedulePlan = 209
    SupportTimezone = 46
    SupportTipsVoice = 625
    SupportTracking = 198
    SupportTvEntranceOff = 578
    SupportUnLock = 78
    SupportUnbind = 44
    SupportUpgrade = 10
    SupportUploadCloudFile = 18
    SupportVerticalPanoramic = 112
    SupportVideoJoint = 782
    SupportVideoJointLineType = 787
    SupportVideoMeeting = 818
    SupportVideoMeetingEncodeType = 864
    SupportVideoMeetingScreenShare = 867
    SupportVolumnSet = 75
    SupportWeixin = 24
    SupportWifi = 13
    SupportWifi24G = 41
    SupportWifi5G = 42
    SupportWifiLock = 541
    SupportWifiManager = 239
    SupportWifiPortal = 43
    SupportWindowPtzSlider = 802
    SupportWorkModeList = 502
    SupportSensitiveUnderDefenceType = 444
    SupportDefenceTypeFull = 534
    SupportDetectAreaUnderDefencetype = 504


@unique
class SoundMode(Enum):
    """Alarm sound level description."""

    SILENT = 2
    SOFT = 0
    INTENSE = 1
    CUSTOM = 3
    PLAN = 4
    UNKNOWN = -1


@unique
class DefenseModeType(Enum):
    """Defense mode name and number."""

    HOME_MODE = 1
    AWAY_MODE = 2
    SLEEP_MODE = 3
    UNSET_MODE = 0


@unique
class AlarmDetectHumanCar(Enum):
    """Detection modes for cameras that support AlarmDetectHumanCar."""

    DETECTION_MODE_HUMAN_SHAPE = 1
    DETECTION_MODE_PIR = 5
    DETECTION_MODE_IMAGE_CHANGE = 3


@unique
class IntelligentDetectionSmartApp(Enum):
    """Intelligent detection modes for cameras using smart apps."""

    app_human_detect = 1
    app_video_change = 4
    app_car_detect = 2
    app_wave_recognize = 64


@unique
class NightVisionMode(Enum):
    """Intelligent detection modes."""

    NIGHT_VISION_COLOUR = 1
    NIGHT_VISION_B_W = 0
    NIGHT_VISION_SMART = 2


@unique
class DisplayMode(Enum):
    """Display modes or image styles."""

    DISPLAY_MODE_ORIGINAL = 1
    DISPLAY_MODE_SOFT = 2
    DISPLAY_MODE_VIVID = 3


@unique
class BatteryCameraWorkMode(Enum):
    """Battery camera work modes."""

    UNKNOWN = -1
    POWER_SAVE = 0
    HIGH_PERFORMANCE = 1
    PLUGGED_IN = 2
    SUPER_POWER_SAVE = 3
    CUSTOM = 4
    HYBERNATE = 5  # not sure


@unique
class BatteryCameraNewWorkMode(Enum):
    """New battery camera work modes."""

    UNKNOWN = -1
    STANDARD = 1
    PLUGGED_IN = 2
    SUPER_POWER_SAVE = 3
    CUSTOM = 4
    ALWAYS_ON_VIDEO = 7


class DeviceCatagories(Enum):
    """Supported device categories."""

    COMMON_DEVICE_CATEGORY = "COMMON"
    CAMERA_DEVICE_CATEGORY = "IPC"
    BATTERY_CAMERA_DEVICE_CATEGORY = "BatteryCamera"
    DOORBELL_DEVICE_CATEGORY = "BDoorBell"
    BASE_STATION_DEVICE_CATEGORY = "XVR"
    CAT_EYE_CATEGORY = "CatEye"
    LIGHTING = "lighting"



"""pyezvizapi CAS API Functions."""

from __future__ import annotations

from io import BytesIO
from itertools import cycle
import logging
import random
import socket
import ssl
from typing import Any, cast

from Crypto.Cipher import AES
import xmltodict

from .constants import FEATURE_CODE, XOR_KEY
from .exceptions import InvalidHost, PyEzvizError

_LOGGER = logging.getLogger(__name__)


def xor_enc_dec(msg: bytes, xor_key: bytes = XOR_KEY) -> bytes:
    """XOR encode/decode bytes with the given key."""
    with BytesIO(msg) as stream:
        return bytes(a ^ b for a, b in zip(stream.read(), cycle(xor_key)))


class EzvizCAS:
    """Ezviz CAS server client."""

    def __init__(self, token: dict[str, Any] | None) -> None:
        """Initialize the client object."""
        self._session = None
        self._token: dict[str, Any] = token or {
            "session_id": None,
            "rf_session_id": None,
            "username": None,
            "api_url": "apiieu.ezvizlife.com",
        }
        if not token or "service_urls" not in token:
            raise PyEzvizError("Missing service_urls in token; call EzvizClient.login() first")
        self._service_urls: dict[str, Any] = token["service_urls"]

    def cas_get_encryption(self, devserial: str) -> dict[str, Any]:
        """Fetch encryption code from EZVIZ CAS server."""
        # Random hex 64 characters long.
        rand_hex_str = f"{random.randrange(10**80):064x}"[:64]

        payload = (
            f"\x9e\xba\xac\xe9\x01\x00\x00\x00\x00\x00"
            f"\x00\x02"  # Check or order?
            f"\x00\x00\x00\x00\x00\x00 "
            f"\x01"  # Check or order?
            f"\x00\x00\x00\x00\x00\x00\x02\t\x00\x00\x00\x00"
            f'<?xml version="1.0" encoding="utf-8"?>\n<Request>\n\t'
            f'<ClientID>{self._token["session_id"]}</ClientID>'
            f"\n\t<Sign>{FEATURE_CODE}</Sign>\n\t"
            f"<DevSerial>{devserial}</DevSerial>"
            f"\n\t<ClientType>0</ClientType>\n</Request>\n"
        ).encode("latin1")

        payload_end_padding = rand_hex_str.encode("latin1")

        context = ssl.SSLContext(ssl.PROTOCOL_TLS)
        context.set_ciphers(
            "DEFAULT:!aNULL:!eNULL:!MD5:!3DES:!DES:!RC4:!IDEA:!SEED:!aDSS:!SRP:!PSK"
        )

        # Create a TCP/IP socket
        host = cast(str, self._service_urls["sysConf"][15])
        port = cast(int, self._service_urls["sysConf"][16])
        my_socket = socket.create_connection((host, port))
        my_socket = context.wrap_socket(
            my_socket, server_hostname=self._service_urls["sysConf"][15]
        )

        # Get CAS Encryption Key
        try:
            my_socket.send(payload + payload_end_padding)
            response_bytes = my_socket.recv(1024)
            _LOGGER.debug("Get Encryption Key: %r", response_bytes)
        except (socket.gaierror, ConnectionRefusedError) as err:
            raise InvalidHost("Invalid IP or Hostname") from err
        finally:
            my_socket.close()

        # Trim header, tail and convert xml to dict.
        body = response_bytes[32:-32]
        doc = xmltodict.parse(body)
        return cast(dict[str, Any], doc)

    def set_camera_defence_state(self, serial: str, enable: int = 1) -> bool:
        """Enable alarm notifications."""
        # Random hex 64 characters long.
        rand_hex_str = f"{random.randrange(10**80):064x}"[:64]

        payload = (
            f"\x9e\xba\xac\xe9\x01\x00\x00\x00\x00\x00"
            f"\x00\x14"  # Check or order?
            f"\x00\x00\x00\x00\x00\x00 "
            f"\x05"
            f"\x00\x00\x00\x00\x00\x00\x02\xd0\x00\x00\x01\xe0"
            f'<?xml version="1.0" encoding="utf-8"?>\n<Request>\n\t'
            f'<Verify ClientSession="{self._token["session_id"]}" '
            f'ToDevice="{serial}" ClientType="0" />\n\t'
            f'<Message Length="240" />\n</Request>\n'
            f"\x9e\xba\xac\xe9\x01\x00\x00\x00\x00\x00"
            f"\x00\x13"
            f"\x00\x00\x00\x00\x00\x000\x0f\xff\xff\xff\xff"
            f"\x00\x00\x00\xb0\x00\x00\x00\x00"
        ).encode("latin1")

        payload_end_padding = rand_hex_str.encode("latin1")

        # xor camera serial
        xor_cam_serial = xor_enc_dec(serial.encode("latin1"))

        defence_msg_string = (
            f'{xor_cam_serial.decode()}2+,*xdv.0" '
            f'encoding="utf-8"?>\n'
            f"<Request>\n"
            f"\t<OperationCode>ABCDEFG</OperationCode>\n"
            f'\t<Defence Type="Global" Status="{enable}" Actor="V" Channel="0" />\n'
            f"</Request>\n"
            f"\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10\x10"
        ).encode("latin1")

        context = ssl.SSLContext(ssl.PROTOCOL_TLS)
        context.set_ciphers(
            "DEFAULT:!aNULL:!eNULL:!MD5:!3DES:!DES:!RC4:!IDEA:!SEED:!aDSS:!SRP:!PSK"
        )

        # Create a TCP/IP socket
        host = cast(str, self._service_urls["sysConf"][15])
        port = cast(int, self._service_urls["sysConf"][16])
        my_socket = socket.create_connection((host, port))
        my_socket = context.wrap_socket(
            my_socket, server_hostname=self._service_urls["sysConf"][15]
        )

        cas_client = self.cas_get_encryption(serial)

        aes_key = cas_client["Response"]["Session"]["@Key"].encode("latin1")
        iv_value = (
            f"{serial}{cas_client['Response']['Session']['@OperationCode']}".encode(
                "latin1"
            )
        )

        # Message encryption
        cipher = AES.new(aes_key, AES.MODE_CBC, iv_value)

        try:
            enc_bytes = cipher.encrypt(defence_msg_string)
            my_socket.send(payload + enc_bytes + payload_end_padding)
            _LOGGER.debug("Set camera response: %r", my_socket.recv())
        except (socket.gaierror, ConnectionRefusedError) as err:
            raise InvalidHost("Invalid IP or Hostname") from err
        finally:
            my_socket.close()

        return True



"""Test camera RTSP authentication."""

from __future__ import annotations

import base64
import hashlib
import socket
from typing import TypedDict

from .exceptions import AuthTestResultFailed, InvalidHost


def genmsg_describe(url: str, seq: int, user_agent: str, auth_seq: str) -> str:
    """Generate RTSP DESCRIBE request message."""
    msg_ret = f"DESCRIBE {url} RTSP/1.0\r\n"
    msg_ret += f"CSeq: {seq}\r\n"
    msg_ret += f"Authorization: {auth_seq}\r\n"
    msg_ret += f"User-Agent: {user_agent}\r\n"
    msg_ret += "Accept: application/sdp\r\n\r\n"
    return msg_ret


class RTSPDetails(TypedDict):
    """Typed structure for RTSP test parameters."""
    bufLen: int
    defaultServerIp: str
    defaultServerPort: int
    defaultTestUri: str
    defaultUserAgent: str
    defaultUsername: str | None
    defaultPassword: str | None


class TestRTSPAuth:
    """Test RTSP credentials against an RTSP server."""

    _rtsp_details: RTSPDetails

    def __init__(
        self, ip_addr: str, username: str | None = None, password: str | None = None, test_uri: str = ""
    ) -> None:
        """Initialize RTSP credential test."""
        self._rtsp_details = RTSPDetails(
            bufLen=1024,
            defaultServerIp=ip_addr,
            defaultServerPort=554,
            defaultTestUri=test_uri,
            defaultUserAgent="RTSP Client",
            defaultUsername=username,
            defaultPassword=password,
        )

    def generate_auth_string(self, realm: bytes, method: str, uri: str, nonce: bytes) -> str:
        """Generate the HTTP Digest Authorization header value."""
        m_1 = hashlib.md5(
            f"{self._rtsp_details['defaultUsername']}:{realm.decode()}:{self._rtsp_details['defaultPassword']}".encode()
        ).hexdigest()
        m_2 = hashlib.md5(f"{method}:{uri}".encode()).hexdigest()
        response = hashlib.md5(f"{m_1}:{nonce.decode()}:{m_2}".encode()).hexdigest()

        return (
            "Digest "
            f"username=\"{self._rtsp_details['defaultUsername']}\", "
            f"realm=\"{realm.decode()}\", "
            'algorithm="MD5", '
            f"nonce=\"{nonce.decode()}\", "
            f"uri=\"{uri}\", "
            f"response=\"{response}\""
        )

    def main(self) -> None:
        """Open RTSP socket, try Basic and then Digest auth for DESCRIBE."""
        session: socket.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        try:
            session.connect(
                (self._rtsp_details["defaultServerIp"], self._rtsp_details["defaultServerPort"])
            )
        except TimeoutError as err:
            raise AuthTestResultFailed("Invalid ip or camera hibernating") from err
        except (socket.gaierror, ConnectionRefusedError) as err:
            raise InvalidHost("Invalid IP or Hostname") from err

        seq: int = 1

        url: str = "rtsp://" + self._rtsp_details["defaultServerIp"] + self._rtsp_details["defaultTestUri"]

        # Basic Authorization header
        auth_b64: bytes = base64.b64encode(
            f"{self._rtsp_details['defaultUsername']}:{self._rtsp_details['defaultPassword']}".encode("ascii")
        )
        auth_seq: str = "Basic " + auth_b64.decode()

        describe = genmsg_describe(url, seq, self._rtsp_details["defaultUserAgent"], auth_seq)
        print(describe)
        session.send(describe.encode())
        msg1: bytes = session.recv(self._rtsp_details["bufLen"])
        seq += 1

        decoded = msg1.decode()
        if "200 OK" in decoded:
            print(f"Basic auth result: {decoded}")
            print("Basic Auth test passed. Credentials Valid!")
            return

        if "Unauthorized" in decoded:
            # Basic failed, do new DESCRIBE with digest authentication.
            start = decoded.find("realm")
            begin = decoded.find('"', start)
            end = decoded.find('"', begin + 1)
            realm: bytes = msg1[begin + 1 : end]

            start = decoded.find("nonce")
            begin = decoded.find('"', start)
            end = decoded.find('"', begin + 1)
            nonce: bytes = msg1[begin + 1 : end]

            auth_seq = self.generate_auth_string(realm, "DESCRIBE", self._rtsp_details["defaultTestUri"], nonce)

            describe = genmsg_describe(url, seq, self._rtsp_details["defaultUserAgent"], auth_seq)
            print(describe)
            session.send(describe.encode())
            msg1 = session.recv(self._rtsp_details["bufLen"])
            decoded = msg1.decode()
            print(f"Digest auth result: {decoded}")

            if "200 OK" in decoded:
                print("Digest Auth test Passed. Credentials Valid!")
                return

            if "401 Unauthorized" in decoded:
                raise AuthTestResultFailed("Credentials not valid!!")

        print("Basic Auth test passed. Credentials Valid!")
# ruff: noqa: T201



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='pyezvizapi',
    version="1.0.1.9",
    license='Apache Software License 2.0',
    author='Renier Moorcroft',
    author_email='RenierM26@users.github.com',
    description='Pilot your Ezviz cameras',
    long_description="Pilot your Ezviz cameras with this module. Please view readme on github",
    url='https://github.com/RenierM26/pyEzvizApi/',
    packages=setuptools.find_packages(),
    setup_requires=[
        'requests',
        'setuptools'
    ],
    install_requires=[
        'requests',
        'pandas',
        'paho-mqtt',
        'xmltodict',
        'pycryptodome'
    ],
    entry_points={
    'console_scripts': ['pyezvizapi = pyezvizapi.__main__:main']
    },
    python_requires = '>=3.11'
)
